#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "mex.h"

int compar(const void* ip1, const void* ip2) {

	double* p1 = (double*)ip1;
	double* p2 = (double*)ip2;

	if (*p1 > *p2)
		return -1;
	else if (*p1 < *p2)
		return 1;
	else
		return 0;

}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	// Input
	double* x = mxGetPr(prhs[0]);
	double alpha = mxGetScalar(prhs[1]);
	int n = mxGetNumberOfElements(prhs[0]);

	// Output
	plhs[0] = mxCreateDoubleMatrix(1, n, mxREAL);
	double* y = mxGetPr(plhs[0]);

	if (alpha <= 1.0/n) {
		for(int i = 0; i < n; i++)
			y[i] = 1.0/n;
		return;
	}

	// sort the input and scan through it; find the optimal threshold
	double* z = new double[n];
	memcpy(z, x, sizeof(double) * n);
	qsort(z, n, sizeof(double), compar); // sort z in descending order

	if (z[0] <= alpha) {
		memcpy(y, x, sizeof(double) * n);
		return;
	}

	double thresh = 1e100; // cut everything above this threshold to alpha
	double opt_KL = 1e100;
	double opt_C = 1e100;
	double KL = 0;
	double sum_zi = 1.0;

	KL += alpha * log(alpha / z[0]);
	sum_zi -= z[0];

	for(int i = 1; i < n; i++) {
		double C = (1.0 - alpha*i) / sum_zi;
		if (C <= 0) break;
		if (C * z[i] <= alpha) {
			double tot_KL = KL + C*log(C) * sum_zi;
			if (tot_KL < opt_KL) {
				opt_KL = tot_KL;
				thresh = z[i-1];
				opt_C = C;
			}
		}
		KL += alpha * log(alpha/z[i]);
		sum_zi -= z[i];
	}

	// produce the output
	for(int i = 0; i < n; i++) {
		if (x[i] >= thresh)
			y[i] = alpha;
		else
			y[i] = x[i] * opt_C;
	}

	delete[] z;

}