#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include "tools.h"

#define MAX(x,y) ((x)>(y)?(x):(y))

#ifdef GREEDY_DEBUG_
	#include <stdio.h>
	#define mexPrintf printf
	#define mexEvalString printf
#else
	#include "mex.h"
#endif

void read_data(FILE* f, double* A, int len) {

	int i;
	double t;

	for(i = 0; i < len; i++) {
		fscanf(f, "%lf\n", &t);
		A[i] = t;
	}

}

void print_matrix(double* A, int n, int m) {

	int i, j;

	for(i = 0; i < n; i++) {
		for(j = 0; j < m; j++)
			printf("%lf ", A[i*m+j]);
		printf("\n");
	}

}

/* greedy(objtype, ret, X, A, invA, &logdetA, cov0, n, p, k, verbose_step) */
void greedy(char objtype, int* ret, double* X, double* A, double* invA, double* logdetA, double* cov0, int n, int p, int k, int verbose_step) {

	int* flags = (int*)(malloc(n * sizeof(int)));
	double* workspace = (double*)(malloc(2*p * sizeof(double)));

	int i, j, iter;
	double f_opt = compute_objective(A, invA, *logdetA, p, objtype, cov0, X, n, workspace);
	double f_now;
	int i_opt;

	/* initialization: the full set */
	for(i = 0; i < n; i++)
		flags[i] = 1;

	/* remove one data point at a time */
	for(iter = 0; iter < n-k; iter++) {

		f_opt = 1e100;
		i_opt = -1;
		for(i = 0; i < n; i++)
			if (flags[i]) {
				int success = rank1_update(A, invA, logdetA, X + i * p, -1.0, p, workspace);
				if (!success) continue;
				f_now = compute_objective(A, invA, *logdetA, p, objtype, cov0, X, n, workspace);
				rank1_update(A, invA, logdetA, X + i * p, 1.0, p, workspace);
				if (f_now < f_opt) {
					f_opt = f_now;
					i_opt = i;
				}
			}

		if (i_opt == -1) break;

		rank1_update(A, invA, logdetA, X + i_opt * p, -1.0, p, workspace);
		flags[i_opt] = 0;

		if (verbose_step > 0 && iter % verbose_step == 0) {
			mexPrintf("k = %d: f_opt = %lf\n", n-iter-1, f_opt);
			mexEvalString("pause(.001);");
		}

	}

	/* record the final solution */
	j = 0;
	for(i = 0; i < n; i++)
		if (flags[i]) {
			ret[j++] = i;
			if (j == k) break;
		}

	/* release allocated memory */
	free(flags);
	free(workspace);

}


/* objtype, X, c0, A, invA, logdetA, cov0, withrep/worep, verbose 
c0 is a 1 x k vector, corresponding to an initial set
A = X_c0 * X_c0^T
return: either 1 or 2 variables; C and # of iterations
*/
#ifndef GREEDY_DEBUG_
void mexFunction(int nlhs, mxArray* plhs[], int nrhs, const mxArray *prhs[]) {

	char* objtype = (char*)mxGetChars(prhs[0]);
	mexPrintf("objtype = %c\n", objtype[0]);
	mexEvalString("pause(.001);");
	double* X = mxGetPr(prhs[1]);
	int k = (int)(mxGetScalar(prhs[2]));
	double* A = mxGetPr(prhs[3]);
	double* invA = mxGetPr(prhs[4]);
	double logdetA = mxGetScalar(prhs[5]);
	double* cov0 = mxGetPr(prhs[6]);
	int verbose_step = (int)(mxGetScalar(prhs[7]));

	int p = (int)(mxGetM(prhs[1]));
	int n = (int)(mxGetN(prhs[1]));
	int iters = 0;

	int* ret;
	int i, j;

	plhs[0] = mxCreateNumericMatrix(1, k, mxINT32_CLASS, mxREAL);
	ret = (int*)mxGetData(plhs[0]);

	greedy(objtype[0], ret, X, A, invA, &logdetA, cov0, n, p, k, verbose_step);
	
	for(i = 0; i < k; i++) {
		ret[i] = ret[i] + 1;
	}
	
}
#endif

#ifdef GREEDY_DEBUG_
int main() {

	int n = 100;
	int p = 10;
	int k = 30;

	double* X = (double*)(malloc(n*p * sizeof(double)));
	double* A = (double*)(malloc(p*p * sizeof(double)));
	double* invA = (double*)(malloc(p*p * sizeof(double)));
	double logdetA;
	double* cov0 = (double*)(malloc(p*p * sizeof(double)));

	int* inds = (int*)(malloc(k * sizeof(int)));

	FILE* f = fopen("input.txt", "r");

	int i;
	int iters = 0;

	read_data(f, X, n*p);
	read_data(f, A, p*p);
	read_data(f, invA, p*p);
	read_data(f, &logdetA, 1);
	read_data(f, cov0, p*p);

	fclose(f);

	greedy('E', inds, X, A, invA, &logdetA, cov0, n, p, k, 1);

	for(i = 0; i < k; i++)
		printf("%d ", inds[i] + 1);
	printf("\n");

	return 0;

}
#endif