function [ C ] = pipage( X, k, objtype, w0, verbose_step )
% pipage rounding algorithm. See fedorov.m for explanation of arguments
% except for w0, which is an nx1 vector returned by solve_pi_pgd or
% solve_pi_sgd routines

    if (nargin < 4)
        verbose_step = 1;
    end
    
    w0 = w0 / sum(w0) * k;
    if (max(w0) >= 1+1e-7)
        fprintf('Oops, max(w0) = %f\n', max(w0));
    end
    
    n = size(X, 1);
    A = X'*diag(w0)*X;
    invA = inv(A);
    logdetA = logdet(A, 'chol');
    cov0 = (X'*X) / n;
    
    C = pipage_c(objtype, X', k, A, invA, logdetA, cov0, w0, verbose_step);

end

