function [ w, tot_iters, f_opt ] = solve_pi_psg( X, gamma, infty_norm, objtype, lambda, T, verbose_step )
% See solve_pi_pgd.m for details on the parameters
% new parameter gamma: step size at time t is gamma / t
% use select_psg_stepsize.m to select a gamma that works.

    if (nargin < 7)
        verbose_step = 1;
    end
    if (nargin < 6)
        lambda = 0;
    end
    
    n = size(X, 1);
    
    % initialization
    %w = 1/n * ones(n, 1);
    w = rand(n, 1);
    w = w / sum(w);
    
    
    Sigma = X' * diag(w) * X;
    f_opt = compute_objective(X, Sigma, objtype, lambda);
    w_opt = w;
    if (objtype == 'd' || objtype == 'D'); f_opt = exp(f_opt); end
    fprintf('Start %c-optimality: f_0 = %f\n', objtype, f_opt);
    if (objtype == 'd' || objtype == 'D'); f_opt = log(f_opt); end
    
    % subgradient descent
    iter = 0;
    while (iter < T)
        
        grad = compute_gradient(X, Sigma, objtype, lambda); % 1 x n
        gamma_t = gamma / sqrt(iter+1);
        %gamma_t = gamma;
        wt = w - gamma_t * grad;
        if (sum(isnan(wt)) > 0 || max(abs(wt)) > 1e10)
            tot_iters = 0;
            f_opt = inf;
            return;
        end
        w = project_L1LInf(wt, infty_norm, 1, ones(n, 1));
       
        Sigma = X'*diag(w)*X;
        f_now = compute_objective(X, Sigma, objtype, lambda);
        
        iter = iter + 1;
        if (mod(iter, verbose_step) == 0)
            if (objtype == 'd' || objtype == 'D'); f_now = exp(f_now); end
            fprintf('Step %d: objective = %f, best_objective = %f\n', iter, f_now, f_opt);
            if (objtype == 'd' || objtype == 'D'); f_now = log(f_now); end;
        end     
        
        if (f_now < f_opt)
            f_opt = f_now;
            w_opt = w;
        end
        
    end

    w = w_opt;
    fprintf('Completed. # of nonzero weights = %d\n', sum(w > zeros(n, 1)));
    fprintf('Total # of iterations: %d\n', iter);
    fprintf('Final objective: %f\n', f_opt);
    tot_iters = iter;
    
end

